#ifndef LAGRANGE_HPP
#define LAGRANGE_HPP
#include <cmath>
#include <iostream>
#include "complexe.hpp"
#include "rationnel.hpp"
#include "polynome.hpp"

using namespace std;



template <class X > class lagrange : public polynome<X,X> {
protected:
  int np; // nombre de point
  X * points;
  X * fpoints;
public:
  lagrange(): polynome<X,X>(){
    np = 0;
    points = NULL;
    fpoints = NULL;
  }
  lagrange(int nbpoint){
    np = nbpoint;
    this->deg = nbpoint-1;
    this->a = 0.0; this->b = 0.0;
    points = new X[this->np];
    fpoints = new X[this->np];
    for(int i =0; i<this->np;i++){
      points[i] = 0.0;
      fpoints[i] = 0.0;
    }
    this->coefs = new X[this->deg+1];
    for(int i =0; i<this->deg+1;i++){
      this->coefs[i] = X();
    }
  }
  lagrange(const lagrange & x): polynome<X,X> (x) {
    np = x.np;
    if(x.points != NULL){
      points = new X[np];
      fpoints = new X[np];
      for(int i =0; i<np;i++){
  	points[i] = x.points[i];
	fpoints[i] = x.points[i];
      }
    }
  }
  ~lagrange (){
    if(points != NULL){
      delete [] points;
    }
    if(fpoints != NULL){
      delete [] fpoints;
    }
  }
  lagrange construct();
  polynome<X,X> lagrange_k(int);
  int add_points(X * p, int n);
  int add_fpoints(X * pf, int n);
  lagrange & operator = (const lagrange &);
  int testu();
};

template <class X> lagrange<X> lagrange<X>:: construct() {
  if(np != 3){ cout<<" le cas différent de np=3 (degree 2) ne sont pas implémentés"<<endl; exit(0);}
  else {
    polynome<X,X> lag(2);
    polynome<X,X> res(2);
    double a;
    for(int i=0; i<np; i++){
      lag=this->lagrange_k(i);
      res= fpoints[i] * lag + res;
    }
    for(int i=0; i<np; i++){
      a =res.get(i);
      this->set(a,i);
    }
    return *this;
  }
}


template <class X> polynome<X,X> lagrange<X>:: lagrange_k(int i) {
  polynome<X,X> m1(1);
  polynome<X,X> m2(1);
  polynome<X,X> res(2);
  int im, ip;
  X a,b;
  if(np != 3){ cout<<" le cas différent de np=3 (degree 2) ne sont pas implémentés"<<endl; exit(0);}
  else {
    ip = (i+1) % 3; im = (i+2) % 3;
    a = - points[im]/(points[i]-points[im]);
    b = 1.0/(points[i]-points[im]);
    m1.set(a,0);
    m1.set(b,1);
    a = - points[ip]/(points[i]-points[ip]);
    b = 1.0/(points[i]-points[ip]);
    m2.set(a,0);
    m2.set(b,1);
    res= m1*m2;
  }
  return res;
}

template <class X > int lagrange<X>:: add_points( X * p, int n){
  if(n != np){cout<<" Le nombre de points dans le tableau et le polynomes doivent correspondre" <<endl; exit(0); }
  if(p != NULL){
    for(int i =0; i<np;i++){
      points[i] = p[i];
    }
  }
  return 0;
}

template <class X > int lagrange<X>:: add_fpoints( X * pf, int n){
  if(n != np){cout<<" Le nombre de points dans le tableau et le polynomes doivent correspondre" <<endl; exit(0); }
  if(pf != NULL){
    for(int i =0; i<np;i++){
      fpoints[i] = pf[i];
    }
  }
  return 0;
}


template<class X> lagrange<X> & lagrange<X>:: operator = (const lagrange<X> & p){
  if(&p != this){
     this->deg = p.deg;
     this->a = p.a;
     this->b = p.b;
     this->np = p.np;
     this->coefs = NULL;
     if(p.coefs != NULL){
       this->coefs = new X[this->deg+1];
       for(int i =0;i<this->deg+1;i++){ this->coefs[i]=p.coefs[i]; } }
     if(p.points != NULL){
       this->points = new X[this->np];
       this->fpoints = new X[this->np];
       for(int i =0; i<np;i++){
	 this->points[i] = p.points[i];
	 this->fpoints[i] = p.points[i];
       }
     }
  }
  return *this;
}

template <class X> int lagrange<X>:: testu(){
  double points[3]={-0.1,0.0,0.1};
  double fm, f0, fp, fref;
  fm = (-0.1)*(-0.1)+3.0*(-0.1)+1.0;
  f0 = (0.0)*(0.0)+3.0*(0.0)+1.0;
  fp = (0.1)*(0.1)+3.0*(0.1)+1.0;
  fref=  (-0.027)*(-0.027)+3.0*(-0.027)+1.0;
  
  double fpoints[3]={fm,f0,fp};
  lagrange<double> p(3);
  *this = p;
  this->add_points(points,3);
  this->add_fpoints(fpoints,3);
  this->construct();

  if( abs(fref-(*this)(-0.027))< 0.000000001 ) {
    cout<<"Tests unitaires réussis, classe lagrange: 1/1"<<endl;
  } else  {
    cout<<"Tests unitaires réussis, classe lagrange: 0/1"<<endl;
  }
  return 0;
}  
#endif
