#ifndef ORTHOGONAUX_HPP
#define ORTHOGONAUX_HPP
#include <cmath>
#include <iostream>
#include "complexe.hpp"
#include "rationnel.hpp"
#include "polynome.hpp"

using namespace std;


template <class X > class orthogonaux : public polynome<X,X> {
protected:
  int np; // nombre de point
  X * points;
  X * fpoints;
  int type_polynome;
public:
  orthogonaux(): polynome<X,X>(){
    np = 0;
    points = NULL;
    type_polynome =0;
  }
  orthogonaux(int nbpoint,int degree, int type){
    np = nbpoint;
    type_polynome = type;
    this->deg =degree;
    this->a = 0.0; this->b = 0.0;
    points = new X[this->np];
    fpoints = new X[this->np];
    for(int i =0; i<this->np;i++){
      points[i] = 0.0;
      fpoints[i] = 0.0;
    }
    this->coefs = new X[this->deg+1];
    for(int i =0; i<this->deg+1;i++){
      this->coefs[i] = X();
    }
  }
  orthogonaux(const orthogonaux & x): polynome<X,X> (x) {
    np = x.np;
    type_polynome = x.type_polynome;
    if(x.points != NULL){
      points = new X[np];
      fpoints = new X[np];
      for(int i =0; i<np;i++){
  	points[i] = x.points[i];
	fpoints[i] = x.points[i];
      }
    }
  }
  ~orthogonaux (){
    if(points != NULL){
      delete [] points;
    }
    if(fpoints != NULL){
      delete [] fpoints;
    }
  }
  orthogonaux construct();
  X integral(polynome<X,X> &);
  polynome<X,X> orthogonaux_k(int);
  X poids(X); 
  int add_points(X * p, int n);
  int add_fpoints(X * pf, int n);
  orthogonaux & operator = (const orthogonaux &);
  int testu();
};



template <class X > int orthogonaux<X>:: add_points( X * p, int n){
  if(p != NULL){
    for(int i =0; i<np;i++){
      points[i] = p[i];
    }
  }
  return 0;
}

template <class X > int orthogonaux<X>:: add_fpoints( X * pf, int n){
  if(pf != NULL){
    for(int i =0; i<np;i++){
      fpoints[i] = pf[i];
    }
  }
  return 0;
}


template<class X> orthogonaux<X> & orthogonaux<X>:: operator = (const orthogonaux<X> & p){
  if(&p != this){
     this->deg = p.deg;
     this->a = p.a;
     this->b = p.b;
     this->np = p.np;
     this->coefs = NULL;
     this->type_polynome = p.type_polynome;
     if(p.coefs != NULL){
       this->coefs = new X[this->deg+1];
       for(int i =0;i<this->deg+1;i++){ this->coefs[i]=p.coefs[i]; } }
     if(p.points != NULL){
       this->points = new X[this->np];
       this->fpoints = new X[this->np];
       for(int i =0; i<np;i++){
	 this->points[i] = p.points[i];
	 this->fpoints[i] = p.points[i];
       }
     }
  }
  return *this;
}

template <class X> X orthogonaux<X>::integral(polynome<X,X> & p){
  X res = X();
  X xg=X(), xd=X(), fg=X(), fd=X(), loc=X();
  X loc1=X(), loc2=X(), pg=X(), pd=X();

  for(int j=0;j<np-1;j++){
    xg = points[j ];
    xd = points[j+1];
    fg = fpoints[j];
    fd = fpoints[j+1];
    pg = p(xg);  pd = p(xd);
    loc1 = (fg*pg)*this->poids(xg);
    loc2 = (fd*pd)*this->poids(xd);
    loc = 0.5*abs(xd-xg)*(loc1+loc2);
    res = res + loc;
  }
  return res;
}


template <class  X> orthogonaux<X> orthogonaux<X>::construct(){
  polynome<X,X> res;
  X coefk=X(), a=X();

  if(this->deg > 2){ cout<<" les cas < au degree 2 ne sont pas implémentés"<<endl; exit(0);}
  else {
    res = polynome<X,X>(2);
    for(int i=0; i<this->deg+1; i++){
      polynome<X,X> p(i);
      p = this->orthogonaux_k(i);     
      coefk = this->integral(p);
      res= 0.5*(2*i+1) * coefk * p + res;
    }
    for(int i=0; i<this->deg+1; i++){
      a =res.get(i);
      this->set(a,i);
    }
  }
  return *this;
}


template <class X> X orthogonaux<X>::poids(X x){
  X res=0;
  if(type_polynome ==1) { res = 1.0; }
  if(type_polynome ==2) { res = 1.0/sqrt(1.0-x*x); }
  return res;
}

template <class X> polynome<X,X> orthogonaux<X>::orthogonaux_k(int i){
  polynome<X,X> res;
  if(type_polynome ==1) {
    if(i> 2){ cout<<" les cas < 2 ne sont pas implémentés"<<endl; exit(0);}
    else {
      if(i==0)  {
	res = polynome<X,X>(0);
	res.set(1.0,0);
      }
      if(i==1)  {
	res = polynome<X,X>(1);
	res.set(1.0,1);
      }
      if(i==2)  {
	res = polynome<X,X>(2);
      res.set(-0.5,0);
      res.set(3.0/2.0,2);
      }
    }
  }
  if(type_polynome ==2) {
    if(i> 2){ cout<<" les cas < 2 ne sont pas implémentés"<<endl; exit(0);}
    else {
      if(i==0)  {
	res = polynome<X,X>(0);
	res.set(1.0,0);
      }
      if(i==1)  {
	res = polynome<X,X>(1);
	res.set(1.0,1);
      }
      if(i==2)  {
	res = polynome<X,X>(2);
	res.set(-1.0,0);
	res.set(2.0,2);
      }
    }
  }
  return res;
}




template <class X> int orthogonaux<X>:: testu(){
  
  double points[9]={-1.0,-0.75,-0.5-0.25,0.0,0.25,0.5,0.75,1.0};
  double f1, f2, f3,f4,f5,f6,f7,f8,f9, fref;
  f1 = 3.0*(-1.0)+1.0;
  f2 = 3.0*(-0.75)+1.0;
  f3 = 3.0*(-0.5)+1.0;
  f4 = 3.0*(-0.25)+1.0;
  f5 = 3.0*(0.0)+1.0;
  f6 = 3.0*(0.25)+1.0;
  f7 = 3.0*(0.5)+1.0;
  f8 = 3.0*(0.75)+1.0;
  f9 = 3.0*(1.0)+1.0;
  fref= 3.0*(-0.27)+1.0;
  
  double fpoints[9]={f1,f2,f3,f4,f5,f6,f7,f8,f9};

  orthogonaux<double> p(9,2,1);
  
  *this = p;
  this->add_points(points,9);
  this->add_fpoints(fpoints,9);
  this->construct();

  if( abs(fref-(*this)(-0.27))< 0.3 ) {
    cout<<"Tests unitaires réussis, classe orthogonaux: 1/1"<<endl;
  } else  {
    cout<<"Tests unitaires réussis, classe orthogonaux: 0/1"<<endl;
  }
  return 0;
}  
#endif
